@@ -364,7 +364,7 @@ connection_close_error (struct MHD_Connection *connection,
 static int
 try_ready_normal_body (struct MHD_Connection *connection)
 {
-  int ret;
+  ssize_t ret;
   struct MHD_Response *response;
 
   response = connection->response;
@@ -392,11 +392,11 @@ try_ready_normal_body (struct MHD_Connection *connection)
                                 connection->response_write_position));
   if ((0 == ret) &&
       (0 != (connection->daemon->options & MHD_USE_SELECT_INTERNALLY)))
-    mhd_panic (mhd_panic_cls, __FILE__, __LINE__, 
+    mhd_panic (mhd_panic_cls, __FILE__, __LINE__
 #if HAVE_MESSAGES
-	       "API violation"
+	       , "API violation"
 #else
-	       NULL
+	       , NULL
 #endif
 	       );
   if ( (MHD_CONTENT_READER_END_OF_STREAM == ret) ||
@@ -433,7 +433,7 @@ try_ready_normal_body (struct MHD_Connection *connection)
 static int
 try_ready_chunked_body (struct MHD_Connection *connection)
 {
-  int ret;
+  ssize_t ret;
   char *buf;
   struct MHD_Response *response;
   size_t size;
@@ -508,7 +508,7 @@ try_ready_chunked_body (struct MHD_Connection *connection)
     ret = 0xFFFFFF;
   snprintf (cbuf, 
 	    sizeof (cbuf),
-	    "%X\r\n", ret);
+	    "%X\r\n", (unsigned int) ret);
   cblen = strlen (cbuf);
   EXTRA_CHECK (cblen <= sizeof (cbuf));
   memcpy (&connection->write_buffer[sizeof (cbuf) - cblen], cbuf, cblen);
@@ -561,8 +561,8 @@ add_extra_headers (struct MHD_Connection *connection)
                                             MHD_HTTP_HEADER_CONTENT_LENGTH))
     {
       SPRINTF (buf,
-               "%" MHD_LONG_LONG_PRINTF "u",
-	       (unsigned MHD_LONG_LONG) connection->response->total_size);
+               MHD_UNSIGNED_LONG_LONG_PRINTF,
+	       (MHD_UNSIGNED_LONG_LONG) connection->response->total_size);
       MHD_add_response_header (connection->response,
                                MHD_HTTP_HEADER_CONTENT_LENGTH, buf);
     }
@@ -592,9 +592,12 @@ get_date_string (char *date)
   SPRINTF (date,
            "Date: %3s, %02u %3s %04u %02u:%02u:%02u GMT\r\n",
            days[now.tm_wday % 7],
-           now.tm_mday,
+           (unsigned int) now.tm_mday,
            mons[now.tm_mon % 12],
-           1900 + now.tm_year, now.tm_hour, now.tm_min, now.tm_sec);
+           (unsigned int) (1900 + now.tm_year),
+	   (unsigned int) now.tm_hour, 
+	   (unsigned int) now.tm_min, 
+	   (unsigned int) now.tm_sec);
 }
 
 
@@ -1517,11 +1520,11 @@ process_request_body (struct MHD_Connection *connection)
           return;
         }
       if (processed > used)
-        mhd_panic (mhd_panic_cls, __FILE__, __LINE__, 
+        mhd_panic (mhd_panic_cls, __FILE__, __LINE__
 #if HAVE_MESSAGES
-		   "API violation"
+		   , "API violation"
 #else
-		   NULL
+		   , NULL
 #endif
 		   );
       if (processed != 0)
@@ -1637,6 +1640,7 @@ do_write (struct MHD_Connection *connection)
   return MHD_YES;
 }
 
+
 /**
  * Check if we are done sending the write-buffer.
  * If so, transition into "next_state".
@@ -1659,6 +1663,7 @@ check_write_done (struct MHD_Connection *connection,
   return MHD_YES;
 }
 
+
 /**
  * We have received (possibly the beginning of) a line in the
  * header (or footer).  Validate (check for ":") and prepare
@@ -1769,7 +1774,7 @@ static void
 parse_connection_headers (struct MHD_Connection *connection)
 {
   const char *clen;
-  unsigned MHD_LONG_LONG cval;
+  MHD_UNSIGNED_LONG_LONG cval;
   struct MHD_Response *response;
   const char *enc;
   char *end;
