@@ -31,6 +31,12 @@
 #include "memorypool.h"
 #include <limits.h>
 
+#if HAVE_SEARCH_H
+#include <search.h>
+#else
+#include "tsearch.h"
+#endif
+
 #if HTTPS_SUPPORT
 #include "connection_https.h"
 #include <gnutls/gnutls.h>
@@ -386,6 +392,7 @@ recv_tls_adapter (struct MHD_Connection *connection, void *other, size_t i)
 {
   int res;
 
+  connection->tls_read_ready = MHD_NO;
   res = gnutls_record_recv (connection->tls_session, other, i);
   if ( (GNUTLS_E_AGAIN == res) ||
        (GNUTLS_E_INTERRUPTED == res) )
@@ -401,6 +408,8 @@ recv_tls_adapter (struct MHD_Connection *connection, void *other, size_t i)
       errno = EPIPE;
       return res;
     }
+  if (res == i)
+    connection->tls_read_ready = MHD_YES;
   return res;
 }
 
@@ -606,12 +615,17 @@ MHD_handle_connection (void *data)
 	  tv.tv_usec = 0;
 	  tvp = &tv;
 	}
-#ifdef HAVE_POLL_H
-      if (0 == (con->daemon->options & MHD_USE_POLL)) 
-	{
-#else
+#if HTTPS_SUPPORT
+      if (MHD_YES == con->tls_read_ready)
 	{
+	  /* do not block (more data may be inside of TLS buffers waiting for us) */
+	  tv.tv_sec = 0;
+	  tv.tv_usec = 0;
+	  tvp = &tv;
+	}
 #endif
+      if (0 == (con->daemon->options & MHD_USE_POLL))
+	{
 	  /* use select */
 	  FD_ZERO (&rs);
 	  FD_ZERO (&ws);
@@ -632,7 +646,11 @@ MHD_handle_connection (void *data)
 	      break;
 	    }
 	  /* call appropriate connection handler if necessary */
-	  if (FD_ISSET (con->socket_fd, &rs))
+	  if ( (FD_ISSET (con->socket_fd, &rs))
+#if HTTPS_SUPPORT
+		   || (MHD_YES == con->tls_read_ready) 
+#endif
+	       )
 	    con->read_handler (con);
 	  if (FD_ISSET (con->socket_fd, &ws))
 	    con->write_handler (con);
@@ -663,7 +681,11 @@ MHD_handle_connection (void *data)
 #endif
 	      break;
 	    }
-	  if (0 != (p[0].revents & POLLIN)) 
+	  if ( (0 != (p[0].revents & POLLIN)) 
+#if HTTPS_SUPPORT
+	       || (MHD_YES == con->tls_read_ready) 
+#endif
+	       )
 	    con->read_handler (con);        
 	  if (0 != (p[0].revents & POLLOUT)) 
 	    con->write_handler (con);        
@@ -779,6 +801,12 @@ send_param_adapter (struct MHD_Connection *connection,
 }
 
 
+/**
+ * Signature of main function for a thread.
+ */
+typedef void *(*ThreadStartRoutine)(void *cls);
+
+
 /**
  * Create a thread and set the attributes according to our options.
  * 
@@ -791,7 +819,7 @@ send_param_adapter (struct MHD_Connection *connection,
 static int
 create_thread (pthread_t * thread,
 	       const struct MHD_Daemon *daemon,
-	       void *(*start_routine)(void*),
+	       ThreadStartRoutine start_routine,
 	       void *arg)
 {
   pthread_attr_t attr;
@@ -965,12 +993,7 @@ MHD_add_connection (struct MHD_Daemon *daemon,
   /* non-blocking sockets are required on most systems and for GNUtls;
      however, they somehow cause serious problems on CYGWIN (#1824) */
 #ifdef CYGWIN
-  if
-#if HTTPS_SUPPORT
-    (0 != (daemon->options & MHD_USE_SSL))
-#else
-    (0)
-#endif
+  if (0 != (daemon->options & MHD_USE_SSL))
 #endif
   {
     /* make socket non-blocking */
@@ -1149,27 +1172,32 @@ MHD_accept_connection (struct MHD_Daemon *daemon)
   if (MHD_YES == need_fcntl)
   {
     /* make socket non-inheritable */
-#if !WINDOWS
-    flags = fcntl (s, F_GETFD);
-    if ( ( (-1 == flags) ||
-	   ( (flags != (flags | FD_CLOEXEC)) &&
-	     (0 != fcntl (s, F_SETFD, flags | FD_CLOEXEC)) ) ) )
-#else
+#ifdef WINDOWS
     DWORD dwFlags;
     if (!GetHandleInformation ((HANDLE) s, &dwFlags) ||
         ((dwFlags != dwFlags & ~HANDLE_FLAG_INHERIT) &&
         !SetHandleInformation ((HANDLE) s, HANDLE_FLAG_INHERIT, 0)))
-#endif
       {
 #if HAVE_MESSAGES
-#if WINDOWS
         SetErrnoFromWinError (GetLastError ());
-#endif
 	MHD_DLOG (daemon,
 		  "Failed to make socket non-inheritable: %s\n", 
 		  STRERROR (errno));
 #endif
       }
+#else
+    flags = fcntl (s, F_GETFD);
+    if ( ( (-1 == flags) ||
+	   ( (flags != (flags | FD_CLOEXEC)) &&
+	     (0 != fcntl (s, F_SETFD, flags | FD_CLOEXEC)) ) ) )
+      {
+#if HAVE_MESSAGES
+	MHD_DLOG (daemon,
+		  "Failed to make socket non-inheritable: %s\n", 
+		  STRERROR (errno));
+#endif
+      }
+#endif
   }
 #if HAVE_MESSAGES
 #if DEBUG_CONNECT
@@ -1251,7 +1279,7 @@ MHD_cleanup_connections (struct MHD_Daemon *daemon)
  */
 int
 MHD_get_timeout (struct MHD_Daemon *daemon,
-		 unsigned MHD_LONG_LONG *timeout)
+		 MHD_UNSIGNED_LONG_LONG *timeout)
 {
   time_t earliest_deadline;
   time_t now;
@@ -1268,17 +1296,26 @@ MHD_get_timeout (struct MHD_Daemon *daemon,
   have_timeout = MHD_NO;
   for (pos = daemon->connections_head; NULL != pos; pos = pos->next)
     {
-      if (0 != pos->connection_timeout) {
-        if (!have_timeout ||
-	    earliest_deadline > pos->last_activity + pos->connection_timeout)
-          earliest_deadline = pos->last_activity + pos->connection_timeout;
 #if HTTPS_SUPPORT
-        if (  (0 != (daemon->options & MHD_USE_SSL)) &&
-	      (0 != gnutls_record_check_pending (pos->tls_session)) )
+      if (MHD_YES == pos->tls_read_ready)
+	{
 	  earliest_deadline = 0;
+	  have_timeout = MHD_YES;
+	  break;
+	}
 #endif
-        have_timeout = MHD_YES;
-      }
+      if (0 != pos->connection_timeout) 
+	{
+	  if ( (! have_timeout) ||
+	       (earliest_deadline > pos->last_activity + pos->connection_timeout) )
+	    earliest_deadline = pos->last_activity + pos->connection_timeout;
+#if HTTPS_SUPPORT
+	  if (  (0 != (daemon->options & MHD_USE_SSL)) &&
+		(0 != gnutls_record_check_pending (pos->tls_session)) )
+	    earliest_deadline = 0;
+#endif
+	  have_timeout = MHD_YES;
+	}
     }
   if (MHD_NO == have_timeout)
     return MHD_NO;
@@ -1311,7 +1348,7 @@ MHD_select (struct MHD_Daemon *daemon,
   int max;
   struct timeval timeout;
   struct timeval *tv;
-  unsigned MHD_LONG_LONG ltimeout;
+  MHD_UNSIGNED_LONG_LONG ltimeout;
   int ds;
 
   timeout.tv_sec = 0;
@@ -1393,7 +1430,11 @@ MHD_select (struct MHD_Daemon *daemon,
           ds = pos->socket_fd;
           if (ds != -1)
             {
-              if (FD_ISSET (ds, &rs))
+              if ( (FD_ISSET (ds, &rs))
+#if HTTPS_SUPPORT
+		   || (MHD_YES == pos->tls_read_ready) 
+#endif
+		   )
                 pos->read_handler (pos);
               if (FD_ISSET (ds, &ws))
                 pos->write_handler (pos);
@@ -1430,7 +1471,7 @@ MHD_poll_all (struct MHD_Daemon *daemon,
   {
     struct pollfd p[2 + num_connections];
     struct MHD_Pollfd mp;
-    unsigned MHD_LONG_LONG ltimeout;
+    MHD_UNSIGNED_LONG_LONG ltimeout;
     unsigned int i;
     int timeout;
     unsigned int poll_server;
@@ -1779,7 +1820,7 @@ parse_options_va (struct MHD_Daemon *daemon,
           break;
         case MHD_OPTION_THREAD_POOL_SIZE:
           daemon->worker_pool_size = va_arg (ap, unsigned int);
-	  if (daemon->worker_pool_size >= SIZE_MAX / sizeof (struct MHD_Daemon))
+	  if (daemon->worker_pool_size >= (SIZE_MAX / sizeof (struct MHD_Daemon)))
 	    {
 #if HAVE_MESSAGES
 	      MHD_DLOG (daemon,
@@ -1994,7 +2035,7 @@ create_socket (int domain, int type, int protocol)
   int ctype = SOCK_STREAM | sock_cloexec;
   int fd;
   int flags;
-#if WINDOWS
+#ifdef WINDOWS
   DWORD dwFlags;
 #endif
  
@@ -2011,19 +2052,19 @@ create_socket (int domain, int type, int protocol)
   if (0 != sock_cloexec)
     return fd; /* this is it */  
   /* flag was not set during 'socket' call, let's try setting it manually */
-#if !WINDOWS
+#ifndef WINDOWS
   flags = fcntl (fd, F_GETFD);
   if (flags < 0)
 #else
   if (!GetHandleInformation ((HANDLE) fd, &dwFlags))
 #endif
   {
-#if WINDOWS
+#ifdef WINDOWS
     SetErrnoFromWinError (GetLastError ());
 #endif
     return fd; /* good luck */
   }
-#if !WINDOWS
+#ifndef WINDOWS
   if (flags == (flags | FD_CLOEXEC))
     return fd; /* already set */
   flags |= FD_CLOEXEC;
@@ -2034,7 +2075,7 @@ create_socket (int domain, int type, int protocol)
   if (!SetHandleInformation ((HANDLE) fd, HANDLE_FLAG_INHERIT, 0))
 #endif
   {
-#if WINDOWS
+#ifdef WINDOWS
     SetErrnoFromWinError (GetLastError ());
 #endif
     return fd; /* good luck */
@@ -2075,11 +2116,24 @@ MHD_start_daemon_va (unsigned int options,
   int res_thread_create;
   int use_pipe;
 
+#ifndef HAVE_INET6
+  if (0 != (options & MHD_USE_IPv6))
+    return NULL;    
+#endif
+#ifndef HAVE_POLL_H
+  if (0 != (options & MHD_USE_POLL))
+    return NULL;    
+#endif
+#if ! HTTPS_SUPPORT
+  if (0 != (options & MHD_USE_SSL))
+    return NULL;    
+#endif
   if (NULL == dh)
     return NULL;
   if (NULL == (daemon = malloc (sizeof (struct MHD_Daemon))))
     return NULL;
   memset (daemon, 0, sizeof (struct MHD_Daemon));
+  /* try to open listen socket */
 #if HTTPS_SUPPORT
   if (0 != (options & MHD_USE_SSL))
     {
@@ -2102,8 +2156,7 @@ MHD_start_daemon_va (unsigned int options,
   daemon->wpipe[0] = -1;
   daemon->wpipe[1] = -1;
 #if HAVE_MESSAGES
-  daemon->custom_error_log =
-    (void (*)(void *, const char *, va_list)) &vfprintf;
+  daemon->custom_error_log = (MHD_LogCallback) &vfprintf;
   daemon->custom_error_log_cls = stderr;
 #endif
 #ifdef HAVE_LISTEN_SHUTDOWN
@@ -2148,6 +2201,7 @@ MHD_start_daemon_va (unsigned int options,
     }
 #endif
 
+
   if (MHD_YES != parse_options_va (daemon, &servaddr, ap))
     {
 #if HTTPS_SUPPORT
@@ -2158,7 +2212,6 @@ MHD_start_daemon_va (unsigned int options,
       free (daemon);
       return NULL;
     }
-
 #ifdef DAUTH_SUPPORT
   if (daemon->nonce_nc_size > 0) 
     {
@@ -2235,17 +2288,7 @@ MHD_start_daemon_va (unsigned int options,
     {
       /* try to open listen socket */
       if ((options & MHD_USE_IPv6) != 0)
-#if HAVE_INET6
 	socket_fd = create_socket (PF_INET6, SOCK_STREAM, 0);
-#else
-      {
-#if HAVE_MESSAGES
-	MHD_DLOG (daemon, 
-		  "AF_INET6 not supported\n");
-#endif
-	goto free_and_fail;
-      }
-#endif
       else
 	socket_fd = create_socket (PF_INET, SOCK_STREAM, 0);
       if (-1 == socket_fd)
@@ -2459,10 +2502,11 @@ MHD_start_daemon_va (unsigned int options,
 #if HAVE_PLIBC_FD
       if (SOCKET_ERROR ==
 	  ioctlsocket (plibc_fd_get_handle (socket_fd), FIONBIO, &sk_flags))
+        goto thread_failed;
 #else
       if (ioctlsocket (socket_fd, FIONBIO, &sk_flags) == SOCKET_ERROR)
-#endif // PLIBC_FD
         goto thread_failed;
+#endif // PLIBC_FD
 #endif // MINGW
 
       /* Allocate memory for pooled objects */
