@@ -144,9 +144,9 @@ http_ahc (void *cls,
     file = NULL;
   if (file == NULL)
     {
-      response = MHD_create_response_from_data (strlen (EMPTY_PAGE),
-                                                (void *) EMPTY_PAGE,
-                                                MHD_NO, MHD_NO);
+      response = MHD_create_response_from_buffer (strlen (EMPTY_PAGE),
+						  (void *) EMPTY_PAGE,
+						  MHD_RESPMEM_PERSISTENT);
       ret = MHD_queue_response (connection, MHD_HTTP_NOT_FOUND, response);
       MHD_destroy_response (response);
     }
